==> spread-5.0.0.54-py3-none-any.whl: spread_core/bam/dali/managers.py <==
import logging
from datetime import datetime

from spread_core.protocols.dali.device import events

import spread_core.errors.project_errors
import spread_core.mqtt as mqtt
import spread_core.protocols.dali as dali
from .providers import lights, sensors
from ..managers import Manager

DALI1 = 1
DALI2 = 2


class Dali(Manager):
    def __init__(self, p_id, m_id):
        super().__init__(p_id, m_id)
        self._groups_1 = dict()
        self._groups_2 = dict()
        self._server_id = None
        self._on_scan = False
        self._scan_time = None
        self._broadcast = None
        self.dump_topic = None
        self.send_topic = None

    def _sorted_info(self):
        _all = self._providers

        def s(_id):
            if _all[_id].dali_type == DALI2:
                return _all[_id].dali_type, 10-_all[_id].instance._value, _id
            else:
                return _all[_id].dali_type, _id

        return sorted(self._providers, key=s)

    @property
    def scan_time(self):
        if self._scan_time is None:
            self._scan_time = datetime.now()
        return self._scan_time

    def parse(self, data):
        super(Dali, self).parse(data)
        if self.send_interface:
            self.send_interface.set_bus_params(self.bus)

    def add_provider(self, data, manager):
        super(Dali, self).add_provider(data, manager)
        provider = self._providers[data['id']]
        if provider.is_device:
            addr = provider.binding.addr.address
            if addr not in self._addresses:
                self._addresses[addr] = []
            self._addresses[provider.binding.addr.address].append(provider.id)
        elif provider.is_broadcast:
            self._broadcast = provider

    def groups_of(self, p_id):
        res = []
        for _group in self._providers.values():
            if isinstance(_group, lights.DaliLight):
                if _group.is_group:
                    if _group.short_addr.group in self._groups_1:
                        if p_id in self._groups_1[_group.short_addr.group]:
                            res.append(_group)
        return res

    @property
    def broadcast(self):
        return self._broadcast

    def add_to_group(self, dali_type, group_id, provider_id):
        _groups = self._groups_1 if dali_type == 1 else self._groups_2
        if group_id not in _groups:
            _groups[group_id] = []
        if provider_id not in _groups[group_id]:
            _groups[group_id].append(provider_id)

    def get_group_members(self, dali_type, group_id):
        if dali_type == DALI1:
            if group_id in self._groups_1:
                return self._groups_1[group_id].copy()
        else:
            if group_id in self._groups_2:
                return self._groups_2[group_id].copy()
        return []

    def get_all_devices(self, dali_type):
        _res = []
        if dali_type == 1:
            for _provider in self._providers.values():
                if _provider.is_device and _provider.dali_type == dali_type:
                    _res.append(_provider.id)
        return _res

    def remove_from_group(self, dali_type, group_id, provider_id):
        _groups = self._groups_1 if dali_type == 1 else self._groups_2
        if group_id in _groups:
            if provider_id in _groups[group_id]:
                _groups[group_id].remove(provider_id)

    def on_exit(self):
        self._event.clear()
        self._external_cmd.clear()
        super(Dali, self).on_exit()

    def on_external_command(self, command_frame):
        _data_str = ' '.join(hex(b)[2:].rjust(2, '0') for b in command_frame)
        _frame = dali.frame.ForwardFrame(len(command_frame) * 8, command_frame)
        _cmd = dali.command.from_frame(_frame)
        if not isinstance(_cmd, dali.command.Command) or len(command_frame) < 2:
            logging.debug(f'EXTERNAL FRAME [{_data_str}]')
        elif isinstance(_cmd, dali.gear.general._GearCommand):
            # DALI_1
            logging.debug(f'EXTERNAL DALI1 COMMAND [{_data_str}]: {_cmd}')

            if hasattr(_cmd, 'destination'):
                _provider = None
                for _p_id, _p in self._providers.items():
                    if isinstance(_p, lights.DaliLight) and _p.binding.addr == _cmd.destination:
                        _provider = _p

                if _provider:
                    _p_ids = [_provider.id]
                elif isinstance(_cmd.destination, dali.address.Group):
                    _p_ids = self.get_group_members(DALI1, _cmd.destination.group)
                elif isinstance(_cmd.destination, dali.address.Broadcast):
                    _p_ids = self.get_all_devices(DALI1)
                else:
                    _p_ids = []

                for _p_id in _p_ids:
                    if _p_id in self._providers:
                        _provider = self._providers[_p_id]
                        if isinstance(_provider, lights.DaliLight):
                            _provider.on_command_sended(_cmd)
        else:
            # DALI_2
            logging.debug(f'EXTERNAL DALI2 FRAME [{_data_str}] of {_cmd}')

    def on_event(self, event_data):
        event = events.Event.of(event_data)
        providers = []

        if event.scheme_id == dali.device.events.SCHEME_ADDR_INST_TYPE:
            if event.short_addr.address in self._addresses:
                for p_id in self._addresses[event.short_addr.address]:
                    _provider = self._providers[p_id]
                    if isinstance(_provider, sensors.DaliSensor) and _provider.type_id == event.instance_type._value:
                        providers.append(_provider)
                        event.data = _provider.type_id
        elif event.scheme_id == dali.device.events.SCHEME_ADDR_INST_NUMBER:
            if event.short_addr.address in self._addresses:
                for p_id in self._addresses[event.short_addr.address]:
                    _provider = self._providers[p_id]
                    if isinstance(_provider, sensors.DaliSensor) and _provider.get_value(sensors.InstanceIndex) == event.instance_number._value:
                        providers.append(_provider)
                        event.data = _provider.type_id

        logging.debug(f'<- {event}')

        for _p in providers:
            _p.on_event(event)

    def GetScan(self, sig):
        value = {"managerID": self.id,
                 "managerType": self.__class__.__name__,
                 "scanData": dict(devices=[], devices2=[]),
                 "scanIndex": 0,
                 "scanName": "",
                 "scanTime": str(self.scan_time).replace(' ', 'T')}

        for addr, ids in self._addresses.items():
            device2 = None
            for p_id in ids:
                if p_id in self._providers:
                    provider = self._providers[p_id]
                    d = provider.serialize()
                    if d:
                        if isinstance(provider, lights.DaliLight):
                            value['scanData']['devices'].append(d)
                        elif isinstance(provider, sensors.DaliSensor):
                            if device2 is None:
                                device2 = d
                                device2['instances'] = []
                            inst = provider.serialize_instance()
                            if inst:
                                device2['instances'].append(inst)

            if device2:
                value['scanData']['devices2'].append(device2)

        jocket = mqtt.spread.variable.Variable(value, key=sig.key)
        topic = mqtt.spread.topic.Reply(self.project_id, sig.session_id)
        self.publish(topic, jocket, retain=False)

==> spread-5.1.0.48-py3-none-any.whl: spread_core/bam/dali/managers.py <==
import logging
from datetime import datetime

from spread_core.protocols.dali.device import events
import spread_core.mqtt as mqtt
import spread_core.protocols.dali as dali
from .providers import lights, sensors
from ..managers import Manager

DALI1 = 1
DALI2 = 2


class Dali(Manager):
    def __init__(self, p_id, m_id):
        super().__init__(p_id, m_id)
        self._groups_1 = dict()
        self._groups_2 = dict()
        self._server_id = None
        self._on_scan = False
        self._scan_time = None
        self._broadcast = None
        self.dump_topic = None
        self.send_topic = None

    def _sorted_info(self):
        _all = self._providers

        def s(_id):
            if _all[_id].dali_type == DALI2:
                return _all[_id].dali_type, 10-_all[_id].instance._value, _id
            else:
                return _all[_id].dali_type, _id

        return sorted(self._providers, key=s)

    @property
    def scan_time(self):
        if self._scan_time is None:
            self._scan_time = datetime.now()
        return self._scan_time

    def parse(self, data):
        super(Dali, self).parse(data)
        if self.send_interface:
            self.send_interface.set_bus_params(self.bus)

    def add_provider(self, data, manager):
        super(Dali, self).add_provider(data, manager)
        provider = self._providers[data['id']]
        if provider.is_device:
            addr = provider.binding.addr.address
            self._addresses.setdefault(addr, [])
            self._addresses[provider.binding.addr.address].append(provider.id)
        elif provider.is_broadcast:
            self._broadcast = provider

    def groups_of(self, p_id):
        res = []
        for _group in self._providers.values():
            if isinstance(_group, lights.DaliLight):
                if _group.is_group:
                    if _group.short_addr.group in self._groups_1:
                        if p_id in self._groups_1[_group.short_addr.group]:
                            res.append(_group)
        return res

    @property
    def broadcast(self):
        return self._broadcast

    def add_to_group(self, dali_type, group_id, provider_id):
        _groups = self._groups_1 if dali_type == 1 else self._groups_2
        if group_id not in _groups:
            _groups[group_id] = []
        if provider_id not in _groups[group_id]:
            _groups[group_id].append(provider_id)

    def get_group_members(self, dali_type, group_id):
        if dali_type == DALI1:
            if group_id in self._groups_1:
                return self._groups_1[group_id].copy()
        else:
            if group_id in self._groups_2:
                return self._groups_2[group_id].copy()
        return []

    def get_all_devices(self, dali_type):
        _res = []
        if dali_type == 1:
            for _provider in self._providers.values():
                if _provider.is_device and _provider.dali_type == dali_type:
                    _res.append(_provider.id)
        return _res

    def remove_from_group(self, dali_type, group_id, provider_id):
        _groups = self._groups_1 if dali_type == 1 else self._groups_2
        if group_id in _groups:
            if provider_id in _groups[group_id]:
                _groups[group_id].remove(provider_id)

    def on_exit(self):
        self._event.clear()
        self._external_cmd.clear()
        super(Dali, self).on_exit()

    def on_external_command(self, command, response, *args):
        if isinstance(command, (dali.gear.general._StandardCommand, dali.gear.general.DAPC)):
            if isinstance(command.destination, dali.address.Short):
                devices = self.providers_by_address(command.destination.address)
            else:
                devices = self._providers.values()

            for _provider in devices:
                if _provider is None:
                    continue

                if not isinstance(_provider, lights.DaliLight):
                    continue

                if isinstance(command.destination, dali.address.Short) and _provider.short_addr != command.destination:
                    continue

                if isinstance(command.destination, dali.address.Group) and _provider.id not in self._groups_1.get(
                        command.destination.group, []):
                    continue

                if command.is_query:
                    [_provider.on_request_answer(funit_type, response, None, *args) for funit_type in command.funit_type]
                else:
                    _provider.on_command_sended(command)

                # if isinstance(command.destination, dali.address.Short):
                #     break

        elif isinstance(command, dali.device.general._DeviceCommand):
            # DALI_2
            if isinstance(command, dali.device.general._SpecialDeviceCommand):
                pass
            else:
                if isinstance(command.destination, dali.address.Short):
                    devices = self.providers_by_address(command.destination.address)
                else:
                    devices = self._providers.values()

                for _provider in devices:
                    if not _provider.is_device:
                        continue

                    if not isinstance(_provider, sensors.DaliSensor):
                        continue

                    if isinstance(command.destination, dali.address.Short) and _provider.short_addr != command.destination:
                        continue

                    if isinstance(command.destination, dali.address.Group) and _provider.id not in self._groups_2.get(command.destination.group, []):
                        continue

                    if isinstance(command, dali.device.general._StandardInstanceCommand):
                        if not isinstance(command.instance, (dali.address.InstanceGroup, dali.address.InstanceBroadcast)):
                            if not _provider.compare_instance(command.instance):
                                continue

                    if command.is_query:
                        [_provider.on_request_answer(funit_type, response, None, *args) for funit_type in command.funit_type]
                    elif command.is_config:
                        _provider.on_command_sended(command)

                    # if isinstance(command.destination, dali.address.Short):
                    #     break

        # else:
        #     logging.debug('EXTERNAL UNKNOWN FRAME %s, %s, %s' % (command, response, args))

    def on_event(self, event_data):
        event = events.Event.of(event_data)
        providers = []

        if event.scheme_id == dali.device.events.SCHEME_ADDR_INST_TYPE:
            if event.short_addr.address in self._addresses:
                for p_id in self._addresses[event.short_addr.address]:
                    _provider = self._providers[p_id]
                    if isinstance(_provider, sensors.DaliSensor) and _provider.compare_instance(event.instance_type):
                        providers.append(_provider)
                        event.data = _provider.type_id
        elif event.scheme_id == dali.device.events.SCHEME_ADDR_INST_NUMBER:
            if event.short_addr.address in self._addresses:
                for p_id in self._addresses[event.short_addr.address]:
                    _provider = self._providers[p_id]
                    if isinstance(_provider, sensors.DaliSensor) and _provider.compare_instance(event.instance_number):
                        providers.append(_provider)
                        event.data = _provider.type_id

        logging.debug(f'<- {event}')

        for _p in providers:
            _p.on_event(event)

    def GetScan(self, sig):
        value = {"managerID": self.id,
                 "managerType": self.__class__.__name__,
                 "scanData": dict(devices=[], devices2=[]),
                 "scanIndex": 0,
                 "scanName": "",
                 "scanTime": str(self.scan_time).replace(' ', 'T')}

        for addr, ids in self._addresses.items():
            device2 = None
            for p_id in ids:
                if p_id in self._providers:
                    provider = self._providers[p_id]
                    d = provider.serialize()
                    if d:
                        if isinstance(provider, lights.DaliLight):
                            value['scanData']['devices'].append(d)
                        elif isinstance(provider, sensors.DaliSensor):
                            if device2 is None:
                                device2 = d
                                device2['instances'] = []
                            inst = provider.serialize_instance()
                            if inst:
                                device2['instances'].append(inst)

            if device2:
                value['scanData']['devices2'].append(device2)

        jocket = mqtt.spread.variable.Variable(value, key=sig.key)
        topic = mqtt.spread.topic.Reply(self.project_id, sig.session_id)
        self.publish(topic, jocket, retain=False)
